import traverse from '@babel/traverse';
import * as t from '@babel/types';
import { parse } from '@babel/parser';
import { transformFromAst as babelTransformFromAst } from '@babel/core';
import { transformJS } from './transform';
import { defaultBabelConfig } from '../config/default-babel-config';
function extractVariableNames(pattern) {
  var names = [];
  var _traverse = function traverse(node) {
    if (!node) return;
    switch (node.type) {
      case 'Identifier':
        names.push(node.name);
        break;
      case 'ArrayPattern':
        node.elements.forEach(_traverse);
        break;
      case 'ObjectPattern':
        node.properties.forEach(function (prop) {
          if (prop.type === 'ObjectProperty') {
            // 处理 { a: b } 形式
            if (prop.value.type === 'Identifier') {
              names.push(prop.value.name);
            } else {
              _traverse(prop.value);
            }
          }
          // 处理剩余运算符 ...rest
          else if (prop.type === 'RestElement') {
            _traverse(prop.argument);
          }
        });
        break;
      case 'AssignmentPattern':
        _traverse(node.left);
        break;
      case 'RestElement':
        _traverse(node.argument);
        break;
    }
  };
  _traverse(pattern);
  return names;
}

/**
 * get all methods from code-editor-pane
 */
export var statementParser = function statementParser(code, ast) {
  var statements = [];
  var errorsByStatements = {};
  traverse(ast, {
    FunctionDeclaration: function FunctionDeclaration(path) {
      var _path$node$id;
      if (((_path$node$id = path.node.id) === null || _path$node$id === void 0 ? void 0 : _path$node$id.name) !== 'Page') return;
      var body = path.node.body.body;
      body.forEach(function (stmt) {
        var codeLine = code.slice(stmt.start, stmt.end);
        // 处理变量声明（useState）
        if (stmt.type === 'VariableDeclaration') {
          var decl = stmt.declarations[0];
          var node = decl.id;
          var names = extractVariableNames(node);
          statements.push({
            type: 'JsVarDeclaration',
            value: codeLine,
            names: names
          });
          return;
        }

        // 处理钩子调用（useEffect等）
        if (stmt.type === 'ExpressionStatement') {
          statements.push({
            type: 'JSExpression',
            value: codeLine
          });
          return;
        }

        // 处理函数声明
        if (t.isFunctionDeclaration(stmt)) {
          var name = stmt.id.name;
          statements.push({
            type: 'JSFunction',
            name: name,
            value: codeLine,
            source: codeLine
          });
          var _code = parse('');
          _code.program.body.push(stmt);
          var codeStr = babelTransformFromAst(_code).code;
          var _transformJS = transformJS(codeStr, defaultBabelConfig),
            hasError = _transformJS.hasError,
            errorInfo = _transformJS.errorInfo,
            _transformJS$code = _transformJS.code,
            compiledCode = _transformJS$code === void 0 ? '' : _transformJS$code;
          if (hasError && errorInfo) {
            errorsByStatements[name] = errorInfo;
          }
        }
      });
    }
  });
  return {
    statements: statements,
    errorsByStatements: errorsByStatements
  };
};