"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.statementParser = void 0;
var _traverse2 = _interopRequireDefault(require("@babel/traverse"));
var t = _interopRequireWildcard(require("@babel/types"));
var _parser = require("@babel/parser");
var _core = require("@babel/core");
var _transform = require("./transform");
var _defaultBabelConfig = require("../config/default-babel-config");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
function extractVariableNames(pattern) {
  var names = [];
  var _traverse = function traverse(node) {
    if (!node) return;
    switch (node.type) {
      case 'Identifier':
        names.push(node.name);
        break;
      case 'ArrayPattern':
        node.elements.forEach(_traverse);
        break;
      case 'ObjectPattern':
        node.properties.forEach(function (prop) {
          if (prop.type === 'ObjectProperty') {
            // 处理 { a: b } 形式
            if (prop.value.type === 'Identifier') {
              names.push(prop.value.name);
            } else {
              _traverse(prop.value);
            }
          }
          // 处理剩余运算符 ...rest
          else if (prop.type === 'RestElement') {
            _traverse(prop.argument);
          }
        });
        break;
      case 'AssignmentPattern':
        _traverse(node.left);
        break;
      case 'RestElement':
        _traverse(node.argument);
        break;
    }
  };
  _traverse(pattern);
  return names;
}

/**
 * get all methods from code-editor-pane
 */
var statementParser = exports.statementParser = function statementParser(code, ast) {
  var statements = [];
  var errorsByStatements = {};
  (0, _traverse2["default"])(ast, {
    FunctionDeclaration: function FunctionDeclaration(path) {
      var _path$node$id;
      if (((_path$node$id = path.node.id) === null || _path$node$id === void 0 ? void 0 : _path$node$id.name) !== 'Page') return;
      var body = path.node.body.body;
      body.forEach(function (stmt) {
        var codeLine = code.slice(stmt.start, stmt.end);
        // 处理变量声明（useState）
        if (stmt.type === 'VariableDeclaration') {
          var decl = stmt.declarations[0];
          var node = decl.id;
          var names = extractVariableNames(node);
          statements.push({
            type: 'JsVarDeclaration',
            value: codeLine,
            names: names
          });
          return;
        }

        // 处理钩子调用（useEffect等）
        if (stmt.type === 'ExpressionStatement') {
          statements.push({
            type: 'JSExpression',
            value: codeLine
          });
          return;
        }

        // 处理函数声明
        if (t.isFunctionDeclaration(stmt)) {
          var name = stmt.id.name;
          statements.push({
            type: 'JSFunction',
            name: name,
            value: codeLine,
            source: codeLine
          });
          var _code = (0, _parser.parse)('');
          _code.program.body.push(stmt);
          var codeStr = (0, _core.transformFromAst)(_code).code;
          var _transformJS = (0, _transform.transformJS)(codeStr, _defaultBabelConfig.defaultBabelConfig),
            hasError = _transformJS.hasError,
            errorInfo = _transformJS.errorInfo,
            _transformJS$code = _transformJS.code,
            compiledCode = _transformJS$code === void 0 ? '' : _transformJS$code;
          if (hasError && errorInfo) {
            errorsByStatements[name] = errorInfo;
          }
        }
      });
    }
  });
  return {
    statements: statements,
    errorsByStatements: errorsByStatements
  };
};